from knitter import logger
import xlrd


class ExcelSheet:
    def __init__(self, excel, sheet):
        """
        :param excel: Location of Excel. e.g. "C:/Archive/TestExcel.xlsx"
        :param sheet: Sheet Name. e.g. "Sheet1"
        """
        self.excel = xlrd.open_workbook(excel)
        self.sheet = self.excel.sheet_by_name(sheet)
        
        # something like: [['', '', '', '', ''], ['', '', '', '', ''], ['', '', '', '', ''], ['', '', '', '', '']]
        self.data = [["" for x in range(self.sheet.ncols)] for y in range(self.sheet.nrows)] 
        
        for i in range(self.sheet.nrows):
            for j in range(self.sheet.ncols):
                self.data[i][j] = self.cellxy(i, j)

    def nrows(self):
        return self.sheet.nrows
    
    def ncols(self):
        return self.sheet.ncols
    
    def cellxy(self, rowx, colx):
        """
        Description:
            If the cell value is number, 1234 will get as 1234.0, so fix this issue.
        
        Reference:
            http://stackoverflow.com/questions/2739989/reading-numeric-excel-data-as-text-using-xlrd-in-python
            http://www.lexicon.net/sjmachin/xlrd.html  (Search for "ctype")
            
            self.sheet.cell(rowx, colx).ctype:
                Type symbol    Type number    Python value
                XL_CELL_EMPTY    0           empty string u''
                XL_CELL_TEXT     1           a Unicode string
                XL_CELL_NUMBER   2           float
                XL_CELL_DATE     3           float
                XL_CELL_BOOLEAN  4           int; 1 means TRUE, 0 means FALSE
                ......
        """
        
        cell_value = self.sheet.cell(rowx, colx).value
        
        if self.sheet.cell(rowx, colx).ctype in (2, 3) and int(cell_value) == cell_value:
            cell_value = int(cell_value)
        
        return str(cell_value)

    def cell(self, rowx, col_name):
        for colx in range(0, self.ncols()):
            if self.cellxy(0, colx) == col_name:
                logger.step_normal("ExcelSheet.cellx(%s, %s)=[%s]" % (rowx, col_name, self.cellxy(rowx, colx)))
                return self.cellxy(rowx, colx)

        return None

    def cell_by_colname(self, rowx, col_name):
        for colx in range(0, self.sheet.ncols):
            if self.data[0][colx] == col_name:
                logger.step_normal("Excel Column [%s]=[%s]" % (col_name, self.data[rowx][colx]))
                return self.data[rowx][colx]
        
        return None

    def cell_by_rowname(self, row_name, colx):
        for rowx in range(0, self.sheet.nrows):
            if self.data[rowx][0] == row_name:
                logger.step_normal("Excel [row-name:%s, col:%s]=[%s]" % (row_name, colx, self.data[rowx][colx]))
                return self.data[rowx][colx]

        return None

    def cell_by_row_and_col_name(self, row_name, col_name):
        for rowx in range(0, self.nrows()):
            if self.cellxy(rowx, 0) == row_name:
                logger.step_normal("Excel row_name=" + row_name + ", col_name=" + col_name + ", value=" + self.cell(rowx, col_name))
                return self.cell(rowx, col_name)

        return None

















import os
import inspect
import datetime
import sys
import shutil

from knitter.configure import General
from knitter import logger

def get_sub_folder_names(full_path):
    return [ name for name in os.listdir(full_path) if os.path.isdir(os.path.join(full_path, name)) ]


def get_value_from_conf(conf_file, key):

    if not os.path.exists(conf_file):
        return ""

    if not os.path.isfile(conf_file):
        return ""

    try:
        with open(conf_file, 'r') as f:
            while True:
                data = f.readline()

                if not data:
                    break

                if len(data.split('=')) < 2:
                    continue

                if data.strip()[0] == "#":
                    continue

                if data.split('=')[0].strip() == key:
                    return str(data.split('=', 1)[1].strip())
    except IOError:
        return ""


def version_info():
    from knitter import __version__ as knitter_version
    from selenium import __version__ as selenium_version
    from sys import version as python_version

    browser_version = ""
    for k, v in General.VersionInfo.items():
        browser_version += "%s %s, " % (k, v)

    return "Python %s, %sKnitter %s, Selenium %s" % (python_version.split(" ")[0],
                                                                   browser_version, knitter_version, selenium_version)


def timestamp_date():
    return datetime.datetime.now().strftime("%Y-%m-%d")


def timestamp_date_and_time():
    return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")


def timestamp_for_file_name():
    return datetime.datetime.now().strftime("%Y-%m-%d_%H_%M_%S")


def exception_error():
    error_message = ""
    for i in range(len(inspect.trace())):
        error_line = """
File:      %s - [%s]
Function:  %s
Statement: %s
-------------------------------------------------------------------------------------------""" % \
                     (inspect.trace()[i][1], inspect.trace()[i][2], inspect.trace()[i][3], inspect.trace()[i][4])

    error_message = "%s%s" % (error_message, error_line)
    error_message = """Error!
%s
%s
======================================== Error Message ====================================%s

======================================== Error Message ======================================================""" % \
                    (sys.exc_info()[0], sys.exc_info()[1], error_message)

    return error_message


def delete_folder(folder_path):
    if os.path.exists(folder_path):
        shutil.rmtree(folder_path)


def delete_file_or_folder(file_full_path):
    if os.path.exists(file_full_path):
        if os.path.isdir(file_full_path):
            delete_folder(file_full_path)
        else:
            os.remove(file_full_path)


def copy(src, destination):
    try:
        if os.path.isdir(src):
            shutil.copytree(src, destination)
        else:
            shutil.copy(src, destination)
    except Exception as e:
        logger.handle_exception(e)


def create_folder(path):
    if not os.path.exists(path):
        os.makedirs(path)


if __name__ == "__main__":
    version_info()





class Browser:
    StartURL = ""
    RunningBrowser = None
    AvailableBrowsers = []
    HeadlessMode = False

    class FireFox:
        Driver = ""
        Binary = ""

    class Chrome:
        Driver = ""

    class IE:
        Driver = ""


class General:
    class TestCase:
        Name = ""
        Pass = True
        Warnings = ""
        StartTime = ""
        EndTime = ""
        ScreenShot = ""

    class TestModule:
        Name = ""

    class Total:
        StartTime = ""
        EndTime = ""
        NumberOfTestCasePass = 0
        NumberOfTestCaseFail = 0

    class Path:
        Result = ""

    VersionInfo = {}

    HTMLReporterLines = []

    # Only show log in the console, and exit all testing when any case fail.
    QuickTest = False





from datetime import datetime
import os
import sys

from knitter.configure import Browser, General
from knitter import library
from knitter import reportor


def start_project():
    General.TestCase.Browser = None
    General.TestCase.Name = ""
    General.TestCase.Pass = True
    General.TestCase.Warnings = 0
    General.TestCase.StartTime = ""
    General.TestCase.EndTime = ""

    General.TestModule.Name = ""

    General.Total.NumberOfTestCasePass = 0
    General.Total.NumberOfTestCaseFail = 0
    General.Total.StartTime = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    General.HTMLReporterLines[:] = []

    library.delete_folder(os.path.join(General.Path.Result, "testcase"))
    library.delete_folder(os.path.join(General.Path.Result, "screenshots"))

    reportor.generate_html_report([General.Total.StartTime, "N/A", "N/A", "N/A", "N/A", "N/A"], [])

    print(">>>>>>  [%s]  =>  Start Testing......       <<<<<<" % General.Total.StartTime)


def end_project():
    General.Total.EndTime = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    print(">>>>>>  [%s]  =>  Test End. Duration [%s], All [%s], Pass [%s], Fail [%s]       <<<<<<" % (
        General.Total.EndTime,
        datetime.strptime(General.Total.EndTime, "%Y-%m-%d %H:%M:%S") - datetime.strptime(General.Total.StartTime,
                                                                                          "%Y-%m-%d %H:%M:%S"),
        General.Total.NumberOfTestCasePass + General.Total.NumberOfTestCaseFail,
        General.Total.NumberOfTestCasePass,
        General.Total.NumberOfTestCaseFail
    ))

    reportor.generate_html_report([General.Total.StartTime, General.Total.EndTime,
                                   datetime.strptime(General.Total.EndTime, "%Y-%m-%d %H:%M:%S") - datetime.strptime(General.Total.StartTime, "%Y-%m-%d %H:%M:%S"),
                                   General.Total.NumberOfTestCasePass + General.Total.NumberOfTestCaseFail,
                                   General.Total.NumberOfTestCasePass,
                                   General.Total.NumberOfTestCaseFail], General.HTMLReporterLines, countdown=False)
    reportor.save_current_report_to_repository()
    reportor.generate_report_history()
    reportor.generate_html_report([General.Total.StartTime, General.Total.EndTime,
                                   datetime.strptime(General.Total.EndTime, "%Y-%m-%d %H:%M:%S") - datetime.strptime(General.Total.StartTime, "%Y-%m-%d %H:%M:%S"),
                                   General.Total.NumberOfTestCasePass + General.Total.NumberOfTestCaseFail,
                                   General.Total.NumberOfTestCasePass,
                                   General.Total.NumberOfTestCaseFail], General.HTMLReporterLines, countdown=True)

    print("")
    print(library.version_info())


def handle_exception(e):
    if General.TestCase.Pass is False:
        return

    if e is not None:
        step_normal(e)

        if General.QuickTest is True:
            print(library.timestamp_date_and_time() + "                Warning: " + str(e))

    if sys.exc_info()[0] is not None:
        step_warning(library.exception_error())

        if Browser.RunningBrowser is not None:
            image = "Fail__%s__%s__%s.png" % (library.timestamp_for_file_name(), General.TestCase.Name,
                                              Browser.RunningBrowser.name)
            save_screen_shot(image)
            step_normal("Screen short: [%s]" % image)
        else:
            step_normal("No screen shot for browser because browser is not running.")

        General.TestCase.Pass = False


def start_test_case(name, browser):
    General.TestCase.Name = name + "-" + browser.__name__
    General.TestCase.StartTime = datetime.now().replace(microsecond=0)
    General.TestCase.Pass = True
    General.TestCase.Warnings = 0

    write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
              "\n**************  Test Case [%s] [%s]  ***************\n" % (name, browser.__name__))


def stop_test_case():
    General.TestCase.EndTime = datetime.now().replace(microsecond=0)
    General.Total.EndTime = datetime.now().strftime("%Y-%m-%d %H:%M:%S")

    if General.TestCase.Warnings > 0:
        warning_message = ", has [%s] warning(s)!" % General.TestCase.Warnings
    else:
        warning_message = ""

    if General.TestCase.Pass is True:
        print(u"%s    [Pass]  =>  [%s] [%s] [%s] [%s]%s" % (library.timestamp_date_and_time(),
                                                            General.TestCase.EndTime - General.TestCase.StartTime,
                                                            General.TestModule.Name,
                                                            General.TestCase.Name,
                                                            Browser.RunningBrowser.name,
                                                            warning_message))
        General.Total.NumberOfTestCasePass += 1

        General.HTMLReporterLines.append(["%s =&gt; %s" % (General.TestCase.StartTime.strftime("%m-%d %H:%M:%S"),
                                                           General.TestCase.EndTime.strftime("%m-%d %H:%M:%S")),
                                         '<a href="testcase/%s.log">[%s] - %s</a>' % (General.TestCase.Name,
                                                                                      General.TestModule.Name,
                                                                                      General.TestCase.Name),
                                          General.TestCase.EndTime - General.TestCase.StartTime,
                                          Browser.RunningBrowser.name,
                                          '<td>Pass</td>'])

    else:
        print(u"%s    [Fail]  =>  [%s] [%s] [%s] [%s]%s" % (library.timestamp_date_and_time(),
                                                            General.TestCase.EndTime - General.TestCase.StartTime,
                                                            General.TestModule.Name,
                                                            General.TestCase.Name,
                                                            Browser.RunningBrowser.name,
                                                            warning_message))
        General.Total.NumberOfTestCaseFail += 1

        General.HTMLReporterLines.append(["%s =&gt; %s" % (General.TestCase.StartTime.strftime("%m-%d %H:%M:%S"),
                                                           General.TestCase.EndTime.strftime("%m-%d %H:%M:%S")),

                                         '<a href="testcase/%s.log">[%s] - %s</a>' % (General.TestCase.Name,
                                                                                      General.TestModule.Name,
                                                                                      General.TestCase.Name),

                                          General.TestCase.EndTime - General.TestCase.StartTime,
                                          Browser.RunningBrowser.name,

                                          '<td class="tfail"><a href="screenshots/%s">Fail</a></td>' %
                                          General.TestCase.ScreenShot
                                          ])

    reportor.generate_html_report([General.Total.StartTime, General.Total.EndTime,
                                   datetime.strptime(General.Total.EndTime, "%Y-%m-%d %H:%M:%S") -
                                   datetime.strptime(General.Total.StartTime, "%Y-%m-%d %H:%M:%S"),
                                   General.Total.NumberOfTestCasePass + General.Total.NumberOfTestCaseFail,
                                   General.Total.NumberOfTestCasePass,
                                   General.Total.NumberOfTestCaseFail], General.HTMLReporterLines, countdown=True)

    General.TestCase.Pass = True
    General.TestCase.Warnings = 0


def write_log(path, message):
    if General.QuickTest is True:
        return

    log = os.path.join(General.Path.Result, path)
    library.create_folder(os.path.dirname(log))

    with open(log, 'a') as f:
        f.write(message)


def save_screen_shot(image_name):
    if General.QuickTest is True:
        return

    image_path = os.path.join(General.Path.Result, "screenshots")
    library.create_folder(image_path)

    try:
        Browser.RunningBrowser.save_screenshot(os.path.join(image_path, image_name))
        General.TestCase.ScreenShot = image_name
    except Exception as e:
        handle_exception(e)


def step_section(message):
    write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
              "\n%s    Section: %s\n" % (library.timestamp_date_and_time(), message))


def step_normal(message):
    write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
              "%s    Step: %s\n" % (library.timestamp_date_and_time(), message))


def step_pass(message):
    write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
              "%s    Pass: %s\n" % (library.timestamp_date_and_time(), message))


def step_fail(message):
    if Browser.RunningBrowser is not None:
        image_name = "Fail__%s__%s__%s.png" % (library.timestamp_for_file_name(), General.TestCase.Name,
                                               Browser.RunningBrowser.name)

        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "------------ Fail [%s] -------------------\n" % library.timestamp_date_and_time())
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "%s    Fail: %s, Check ScreenShot [%s]\n" % (library.timestamp_date_and_time(), message, image_name))
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "------------ Fail [%s] ---------------------------------------\n" % library.timestamp_date_and_time())

        save_screen_shot(image_name)
    else:
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "------------ Fail [%s] -------------------\n" % library.timestamp_date_and_time())
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "%s    Fail: %s. And the browser not open.\n" % (library.timestamp_date_and_time(), message))
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "------------ Fail [%s] ---------------------------------------\n" % library.timestamp_date_and_time())

    General.TestCase.Pass = False

    if General.QuickTest is True:
        print(library.timestamp_date_and_time() + "                Fail Step: " + message)

    raise AssertionError(message)


def step_warning(message):
    if Browser.RunningBrowser is not None:
        image_name = "Fail__%s__%s__%s.png" % (library.timestamp_for_file_name(), General.TestCase.Name,
                                               Browser.RunningBrowser.name)
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "------------ Warning [%s] -------------------\n" % library.timestamp_date_and_time())
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "%s    Fail: %s, Check ScreenShot [%s]\n" % (library.timestamp_date_and_time(), message, image_name))
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "------------ Warning [%s] ---------------------------------------\n" % library.timestamp_date_and_time())

        save_screen_shot(image_name)
    else:
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "------------ Warning [%s] -------------------\n" % library.timestamp_date_and_time())
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "%s    Fail: %s. And the browser not open.\n" % (library.timestamp_date_and_time(), message))
        write_log(os.path.join("testcase", "%s.log" % General.TestCase.Name),
                  "------------ Warning [%s] ---------------------------------------\n" % library.timestamp_date_and_time())

    General.TestCase.Warnings += 1













"""
HISTORY
    Version 1.0.1:
            - Support Python 3.5 and plus. Python 2.7 is deprecated.
            - Deprecated the multi-threading feature.
            - Rewrote all implementations of executor.
            - Updated HTML report.
            - Updated WebElement methods.
"""
__version__ = "1.0.1"



from selenium import webdriver
from selenium.webdriver.firefox.firefox_binary import FirefoxBinary
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities
from selenium.webdriver.chrome.options import Options

import types
import os
import importlib

from knitter.configure import Browser, General
from knitter import logger


def launch_browser(browser, url):
    """
    Launch and init a new browser, start with the 'url'

    """

    if browser == Browser.FireFox:
        logger.step_normal("Launch Browser FireFox. URL = " + url)

        firefox_capabilities = DesiredCapabilities.FIREFOX

        # disable the download dialogue
        fp = webdriver.FirefoxProfile()
        fp.set_preference('browser.download.manager.showWhenStarting', False)

        try:
            if Browser.FireFox.Binary == "":
                Browser.RunningBrowser = webdriver.Firefox(executable_path=Browser.FireFox.Driver, firefox_profile=fp,
                                                           capabilities=firefox_capabilities,
                                                           log_path=os.path.join(General.Path.Result, "firefox.log"))
            else:
                Browser.RunningBrowser = webdriver.Firefox(executable_path=Browser.FireFox.Driver, firefox_profile=fp,
                                                           firefox_binary=FirefoxBinary(firefox_path=
                                                                                        Browser.FireFox.Binary),
                                                           log_path=os.path.join(General.Path.Result, "firefox.log"))

            if Browser.FireFox.__name__ not in General.VersionInfo:
                General.VersionInfo[Browser.FireFox.__name__] = Browser.RunningBrowser.capabilities['browserVersion']

        except Exception as e:
            logger.handle_exception(e)
            return False

    elif browser == Browser.Chrome:
        try:
            chrome_options = Options()

            if Browser.HeadlessMode is True:
                chrome_options.add_argument("--headless")
                chrome_options.add_argument("--window-size=1920x1080")
                chrome_options.add_argument("--no-sandbox")

            Browser.RunningBrowser = webdriver.Chrome(chrome_options=chrome_options,
                                                      executable_path=Browser.Chrome.Driver)

            if Browser.Chrome.__name__ not in General.VersionInfo:
                General.VersionInfo[Browser.Chrome.__name__] = Browser.RunningBrowser.capabilities['version']
        except Exception as e:
            logger.handle_exception(e)
            return False

    elif browser == Browser.IE:
        '''
        os.popen('TASKKILL /F /IM IEDriverServer.exe')
        dc = DesiredCapabilities.INTERNETEXPLORER.copy()
        dc['nativeEvents'] = False
        dc['acceptSslCerts'] = True
        '''

        try:
            Browser.RunningBrowser = webdriver.Ie(executable_path=Browser.IE.Driver)

            Browser.RunningBrowser.capabilities['acceptInsecureCerts'] = True
            # Browser.RunningBrowser.capabilities['nativeEvents'] = False
            # Browser.RunningBrowser.capabilities['ignoreProtectedModeSettings'] = True

            # print(Browser.RunningBrowser.capabilities)
            if Browser.IE.__name__ not in General.VersionInfo:
                General.VersionInfo[Browser.IE.__name__] = Browser.RunningBrowser.capabilities['browserVersion']
        except Exception as e:
            logger.handle_exception(e)
            return False

    Browser.RunningBrowser.set_window_size(1366, 758)
    Browser.RunningBrowser.set_window_position(0, 0)
    Browser.RunningBrowser.maximize_window()
    Browser.RunningBrowser.set_page_load_timeout(300)
    Browser.RunningBrowser.implicitly_wait(0)

    Browser.RunningBrowser.get(url)

    return True


def quit_browser():
    if Browser.RunningBrowser is not None:
        Browser.RunningBrowser.quit()
        Browser.RunningBrowser = None


def __run_test_module(test_module):
    General.TestModule.Name = test_module.__name__.split('.')[-1]

    cases = []
    for fun in dir(test_module):
        if (not fun.startswith("__")) and (not fun.endswith("__")) and (isinstance(test_module.__dict__.get(fun),
                                                                                   types.FunctionType)):
            if test_module.__dict__.get(fun).__module__ == test_module.__name__:
                cases.append(fun)

    for case in cases:
        if case == 'before_each_case' or case == 'after_each_case' or case == 'before_launch_browser':
            return

        for browser in Browser.AvailableBrowsers:
            try:
                logger.start_test_case(case, browser)

                if hasattr(test_module, 'before_launch_browser'):
                    getattr(test_module, 'before_launch_browser')()

                if hasattr(test_module, 'before_each_case'):
                    getattr(test_module, 'before_each_case')()

                if launch_browser(browser, Browser.StartURL) is False:
                    raise AssertionError("Launch Browser [" + browser.__name__ + "] Fails!")

                getattr(test_module, case)()

                if hasattr(test_module, 'after_each_case'):
                    getattr(test_module, 'after_each_case')()

            except Exception as e:
                logger.handle_exception(e)

                if General.QuickTest is True:
                    return
            finally:
                logger.stop_test_case()
                quit_browser()


def __run_test_case(case):
    test_module = importlib.import_module(case.__module__)
    General.TestModule.Name = case.__module__.split('.')[-1]

    for browser in Browser.AvailableBrowsers:
        try:
            logger.start_test_case(case.__name__, browser)

            if hasattr(test_module, 'before_launch_browser'):
                getattr(test_module, 'before_launch_browser')()

            if hasattr(test_module, 'before_each_case'):
                getattr(test_module, 'before_each_case')()

            if launch_browser(browser, Browser.StartURL) is False:
                raise AssertionError("Launch Browser [" + browser.__name__ + "] Fails!")

            case()

            if hasattr(test_module, 'after_each_case'):
                getattr(test_module, 'after_each_case')()

        except Exception as e:
            logger.handle_exception(e)

            if General.QuickTest is True:
                return
        finally:
            logger.stop_test_case()
            quit_browser()


def wrapper():
    def handle_func(func):
        def handle_args(*args):
            try:
                # args[0] should be a conf definition function.
                args[0]()

                # remove duplicated items.
                Browser.AvailableBrowsers = list(set(Browser.AvailableBrowsers))

                logger.start_project()

                func(*args)

                logger.end_project()

            except Exception as e:
                logger.handle_exception(e)

        return handle_args
    return handle_func


@wrapper()
def run(*args):
    if len(args) < 2:
        print("At least 2 args needed for run(), first as configuration, others are test object(s).")

    for i in range(1, len(args)):
        run_test_object(args[i])


def run_test_object(obj):
    if General.QuickTest is True and General.Total.NumberOfTestCaseFail > 0:
        return

    if isinstance(obj, list):
        for o in obj:
            run_test_object(o)

    elif isinstance(obj, types.ModuleType):
        __run_test_module(obj)

    elif isinstance(obj, types.FunctionType):
        __run_test_case(obj)

    else:
        print("knitter.executor: function [run_test_objects] code error: objects = " + obj)











from setuptools import setup


setup(
    name='knitter',
    version='1.0.0',
    
    author='Henry Wang',
    author_email='outsidematrix@126.com',
    maintainer='Henry Wang',
    maintainer_email='outsidematrix@126.com',
    
    url='https://github.com/hw712/knitter',

    description='A Web Automation Test Framework Based On Selenium WebDriver',
    long_description="Knitter['nitə] is a web automation test framework, with which you can develop "
                     "the web ui automation with good maintainability and extendability.",

    # https://pypi.org/classifiers/
    classifiers=['License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
                 'Topic :: Software Development :: Testing',
                 'Topic :: Software Development :: Quality Assurance',
                 'Topic :: Software Development :: Libraries :: Application Frameworks'],

    platforms=['linux', 'windows'],

    license='BSD License',
    
    packages=['knitter'],
    
    install_requires=['selenium', 'xlrd'],
)














import time
import sys
from selenium import webdriver
from selenium.webdriver.common.keys import Keys
from selenium.common.exceptions import NoSuchElementException, NoAlertPresentException, UnexpectedAlertPresentException
from selenium.common.exceptions import ElementClickInterceptedException

from knitter.configure import Browser
from knitter import logger


class CompatibleMethod(object):
    """If the first argument(self. is a class, set/use properties of the class. If the
    first argument is a instance, set/use properties of the instance.

    This is an extention version for "@classmethod", used for multi-thread issues
    of the framework.

    EXAMPLE

        class A:
            a = 0

            @CompatibleMethod
            def test(self, aaa):
                self.a = aaa

            @CompatibleMethod
            def get(self):
                print "get ", self.a


    """
    def __init__(self, method):
        self._method = method

    def __get__(self, obj, klass=None):
        if klass is None:
            klass = type(obj)

        # this is the different part with "classmethod"
        if isinstance(obj, klass):
            klass = obj
            klass.__name__ = klass.__class__.__name__

        def newfunc(*args, **kws):
            return self._method(klass, *args, **kws)

        return newfunc


class WebBrowser:

    @CompatibleMethod
    def AlertAccept(self):
        logger.step_normal("AlertAccept()")

        time.sleep(2)
        try:
            logger.step_normal("switch_to_alert()")
            alert = Browser.RunningBrowser.switch_to_alert()
            alert.accept()
        except NoAlertPresentException:
            logger.step_normal("Alert Not Found. ")

        try:
            logger.step_normal("switch_to_default_content()")
            Browser.RunningBrowser.switch_to_default_content()
        except Exception as e:
            logger.step_warning(e)
            pass

    @CompatibleMethod
    def AlertDismiss(self):
        logger.step_normal("AlertDismiss()")

        time.sleep(2)
        try:
            logger.step_normal("switch_to_alert()")
            alert = Browser.RunningBrowser.switch_to_alert()
            alert.dismiss()
        except NoAlertPresentException:
            logger.step_normal("Alert Not Found.")

        try:
            logger.step_normal("switch_to_default_content()")
            Browser.RunningBrowser.switch_to_default_content()
        except Exception as e:
            logger.step_normal(e)
            pass

    @CompatibleMethod
    def AlertSendKeys(self, value):
        logger.step_normal("AlertSendKeys [%s]" % value)
        try:
            Browser.RunningBrowser.switch_to.alert.send_keys(value)
            Browser.RunningBrowser.switch_to.default_content()
        except Exception as e:
            logger.step_normal(e)
            logger.step_warning(str(sys.exc_info()))

    @CompatibleMethod
    def AlertTextHave(self, txt_value):
        logger.step_normal("AlertTextHave [%s]" % txt_value)
        alert_text = Browser.RunningBrowser.switch_to_alert().text()

        if txt_value in alert_text:
            logger.step_pass("pass")
        else:
            logger.step_fail("fail")

        Browser.RunningBrowser.switch_to_default_content()

    @CompatibleMethod
    def DeleteAllCookies(self):
        logger.step_normal("Element [%s]: Browser Delete All Cookies" % (self.__name__,))
        Browser.RunningBrowser.delete_all_cookies()

        time.sleep(3)

    @CompatibleMethod
    def IESkipCertError(self):
        logger.step_normal("IE Skip SSL Cert Error.")
        Browser.RunningBrowser.get("javascript:document.getElementById('overridelink').click();")

    @CompatibleMethod
    def NavigateTo(self, url):
        logger.step_normal("Element [%s]: Navigate To [%s]" % (self.__name__, url))
        Browser.RunningBrowser.get(url)

        time.sleep(3)

    @CompatibleMethod
    def Refresh(self, times=4):
        logger.step_normal("Element [%s]: Browser Refresh" % (self.__name__,))

        for i in range(times):
            action = webdriver.ActionChains(Browser.RunningBrowser)
            action.key_down(Keys.CONTROL).send_keys(Keys.F5).key_up(Keys.CONTROL).perform()
            time.sleep(5)

    @CompatibleMethod
    def ScrollTo(self, x, y):
        logger.step_normal("Element [%s]: Scroll To [%s, %s]" % (self.__name__, x, y))
        Browser.RunningBrowser.execute_script("window.scrollTo(%s, %s);" % (x, y))

    @CompatibleMethod
    def SwitchToDefaultContent(self):
        logger.step_normal("SwitchToDefaultContent()")

        try:
            Browser.RunningBrowser.switch_to.default_content()
        except Exception as e:
            logger.step_normal(e)
            logger.step_warning("Browser.RunningBrowser.switch_to.default_content()")

    @CompatibleMethod
    def SwitchToDefaultWindow(self):
        logger.step_normal("SwitchToDefaultWindow()")

        logger.step_normal("Switch To The Default Window of: %s" % str(Browser.RunningBrowser.window_handles))

        try:
            Browser.RunningBrowser.switch_to.window(Browser.RunningBrowser.window_handles[0])
        except Exception as e:
            logger.step_normal(e)
            logger.step_warning("Browser.RunningBrowser.switch_to.window(Browser.RunningBrowser.window_handles[0])")

    @CompatibleMethod
    def SwitchToFrame(self, frame):
        logger.step_normal("SwitchToFrame()")

        Browser.RunningBrowser.switch_to.frame(frame)

    @CompatibleMethod
    def SwitchToNewPopWindow(self):
        logger.step_normal("SwitchToNewPopWindow()")

        t = 0
        while t < 10:
            t = t + 1
            time.sleep(3)

            if len(Browser.RunningBrowser.window_handles) < 2:
                logger.step_normal("Pop Window Not Found. Wait 3 Seconds then Try Again!")
            else:
                break

        Browser.RunningBrowser.switch_to.window(Browser.RunningBrowser.window_handles[-1])

        logger.step_normal("Switch To The New Window of : %s" % str(Browser.RunningBrowser.window_handles))

    @CompatibleMethod
    def Wait(self, seconds):
        logger.step_normal("Element [%s]: Wait for [%s] seconds." % (self.__name__, seconds))
        time.sleep(seconds)


class WebElement:
    (by, value) = (None, None)
    index = 0

    @CompatibleMethod
    def __init__(self, by=None, value=None):
        self.by = by
        self.value = value

    @CompatibleMethod
    def __wait(self):
        t = 0
        while t < 300:
            t = t + 1

            try:
                elements = Browser.RunningBrowser.find_elements(self.by, self.value)
            except NoSuchElementException:
                logger.step_normal("Element [%s]: NoSuchElementException." % self.__name__)
                elements = []
            except UnexpectedAlertPresentException:
                logger.step_warning("Element [%s]: UnexpectedAlertPresentException." % self.__name__)

            if len(elements) == 0:
                time.sleep(1)
                logger.step_normal("Element [%s]: Wait 1 second, By [%s :: %s :: %s]" % (self.__name__,
                                                                                           self.by,
                                                                                           self.value,
                                                                                           self.index))
            else:
                if len(elements) > 1:
                    logger.step_normal("Element [%s]: There are [%s] Elements!" % (self.__name__, len(elements)))

                break

        if len(elements) < self.index + 1:
            logger.step_fail("Element [%s]: Element Index Issue! There are [%s] Elements! Index=[%s]" % (self.__name__,
                                                                                                         len(elements),
                                                                                                         self.index))

    @CompatibleMethod
    def __wait_for_appearing(self):

        t = 0
        while t < 120:
            t = t + 1

            try:
                elements = Browser.RunningBrowser.find_elements(self.by, self.value)
            except NoSuchElementException:
                logger.step_normal("Element [%s]: NoSuchElementException." % self.__name__)
                elements = []
                continue
            except UnexpectedAlertPresentException:
                logger.step_warning("Element [%s]: UnexpectedAlertPresentException." % self.__name__)

            if len(elements) == 0:
                time.sleep(0.5)
                logger.step_normal("Element [%s]: WaitForAppearing... Wait 1 second, By [%s]" % (self.__name__,
                                                                                                 self.value))
            else:
                logger.step_normal("Element [%s]: Found [%s] Element. Tried [%s] Times." % (self.__name__,
                                                                                            len(elements), t))
                break

    @CompatibleMethod
    def __wait_for_disappearing(self):

        t = 0
        while t < 120:
            t = t + 1

            try:
                elements = Browser.RunningBrowser.find_elements(self.by, self.value)
            except NoSuchElementException:
                logger.step_normal("Element [%s]: NoSuchElementException." % self.__name__)
                elements = []
                continue
            except UnexpectedAlertPresentException:
                logger.step_warning("Element [%s]: UnexpectedAlertPresentException." % self.__name__)

            if len(elements) == 0:
                return True
            else:
                time.sleep(0.5)
                logger.step_normal("Element [%s]: WairForDisappearing... Found [%s] Element. Tried [%s] Times." %
                                   (self.__name__, len(elements), t))

        return False

    @CompatibleMethod
    def __wait_for_enabled(self):
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        if elements[self.index].is_enabled():
            return
        else:
            t = 0
            while t < 90:
                if elements[self.index].is_enabled():
                    break

                logger.step_normal("Element [%s]: __wait_for_enabled for 1 second, By [%s :: %s :: %s]" %
                                   (self.__name__, self.by, self.value, self.index))
                time.sleep(1)

    @CompatibleMethod
    def Click(self):
        logger.step_normal("Element [%s]: Click()" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        try:
            elements[self.index].click()
        except ElementClickInterceptedException:
            self.Wait(3)
            action = webdriver.ActionChains(Browser.RunningBrowser)
            action.click(elements[self.index])
            action.perform()

    @CompatibleMethod
    def ClickAndHold(self):
        logger.step_normal("Element [%s]: ClickAndHold()" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        action = webdriver.ActionChains(Browser.RunningBrowser)
        action.click_and_hold(elements[self.index])
        action.perform()

    @CompatibleMethod
    def DoubleClick(self):
        logger.step_normal("Element [%s]: DoubleClick()" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        action = webdriver.ActionChains(Browser.RunningBrowser)
        action.double_click(elements[self.index])
        action.perform()

    @CompatibleMethod
    def DragAndDropByOffset(self, xoffset, yoffset):
        """
        Holds down the left mouse button on the source element,
        then moves to the target offset and releases the mouse button.
        """
        logger.step_normal("Element [%s]: drag_and_drop_by_offset()" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        action = webdriver.ActionChains(Browser.RunningBrowser)
        action.drag_and_drop_by_offset(elements[self.index],xoffset, yoffset)
        action.perform()

    @CompatibleMethod
    def FetchSubElementOfXPath(self, layer):
        return WebElement(self.by, "/".join(self.value.split("/")[:layer+2]))

    @CompatibleMethod
    def GetAttribute(self, attr):
        logger.step_normal("Element [%s]: GetAttribute [%s]." % (self.__name__, attr))

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        attr_value = elements[self.index].get_attribute(attr)
        logger.step_normal("Element [%s]: [%s] = [%s]." % (self.__name__, attr, attr_value))

        return attr_value

    @CompatibleMethod
    def GetInnerHTML(self):
        logger.step_normal("Element [%s]: GetInnerHTML()" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        logger.step_normal("Element [%s]: InnerHTML = [%s]" % (self.__name__, elements[self.index].get_attribute('innerHTML')))

        return elements[self.index].get_attribute('innerHTML')

    @CompatibleMethod
    def GetParentElement(self):
        logger.step_normal("Element [%s]: GetParentElement()" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        return elements[self.index].parent()

    @CompatibleMethod
    def GetRepetition(self):
        logger.step_normal("Element [%s]: GetRepetition()." % self.__name__)

        self.__wait_for_appearing()

        elements = Browser.RunningBrowser.find_elements(self.by, self.value)
        logger.step_normal("Element [%s]: repetition = [%s]" % (self.__name__, len(elements)))

        return len(elements)

    @CompatibleMethod
    def GetRepetitionWithoutWaiting(self):
        """ Get real time obj counts, without waiting."""
        logger.step_normal("Element [%s]: GetRepetitionWithoutWaiting()." % self.__name__)

        elements = Browser.RunningBrowser.find_elements(self.by, self.value)
        logger.step_normal("Element [%s]: repetition = [%s]" % (self.__name__, len(elements)))

        return len(elements)

    @CompatibleMethod
    def GetText(self):
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)
        logger.step_normal("Element [%s]: Get text of the element = %s." % (self.__name__,
                                                                            elements[self.index].text))
        return elements[self.index].text

    @CompatibleMethod
    def IsAttribute(self, attribute, value, assertion="contain"):
        logger.step_normal("Element [%s]: IsAttribute [%s] <%s> [%s]?" % (self.__name__, attribute, assertion, value))

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)
        real_value = elements[self.index].get_attribute(attribute)

        result = False

        if assertion.lower() == 'equal' and value == real_value:
            result = True
        elif assertion.lower() == 'not equal' and value != real_value:
            result = True
        elif assertion.lower() == 'contain' and value in real_value:
            result = True
        elif assertion.lower() == 'not contain' and value not in real_value:
            result = True
        elif assertion.lower() == 'in' and real_value in value:
            result = True
        else:
            logger.step_fail("code error.")

        if result is True:
            logger.step_normal("Yes!")
        else:
            logger.step_normal("No!")

        return result

    @CompatibleMethod
    def IsEnabled(self):
        logger.step_normal("Element [%s]: Is Enabled?" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        if elements[self.index].is_enabled():
            logger.step_normal("Yes!")

            return True
        else:
            logger.step_normal("No!")

            return False

    @CompatibleMethod
    def IsExist(self):
        logger.step_normal("Element [%s]: IsExist?" % self.__name__)

        time.sleep(2)

        elements = Browser.RunningBrowser.find_elements(self.by, self.value)
        logger.step_normal("Element [%s]: IsExist? Count = [%s]" % (self.__name__, len(elements)))

        if len(elements) > 0:
            return True
        else:
            return False

    @CompatibleMethod
    def IsVisible(self):
        logger.step_normal("Element [%s]: IsVisible?" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        if elements[self.index].is_displayed():

            logger.step_normal("Yes!")
            return True
        else:

            logger.step_normal("No!")
            return False

    @CompatibleMethod
    def MouseOver(self):
        logger.step_normal("Element [%s]: MouseOver()" % self.__name__)

        time.sleep(1)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        action = webdriver.ActionChains(Browser.RunningBrowser)
        action.move_to_element(elements[self.index])
        action.perform()

        time.sleep(1)

    @CompatibleMethod
    def ReleaseClick(self):
        logger.step_normal("Element [%s]: ReleaseClick()" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        action = webdriver.ActionChains(Browser.RunningBrowser)
        action.release(elements[self.index])
        action.perform()

    @CompatibleMethod
    def ScrollIntoView(self):
        logger.step_normal("Element [%s]: ScrollToView()" % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        i = 0
        while not elements[self.index].is_displayed():
            WebBrowser.ScrollTo(0, i)
            i = i + 10

            if i == 1000:
                logger.step_normal("still not displayed. break out.")

    @CompatibleMethod
    def Select(self, value):
        logger.step_normal("Element [%s]: Select [%s]." % (self.__name__, value))

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        is_selected = False

        # select
        if elements[self.index].tag_name == "select":
            options = elements[self.index].find_elements_by_tag_name('option')

            for option in options:
                logger.step_normal("Element [%s]: option [%s]" % (self.__name__, option.text))

                if option.text == value:
                    option.click()
                    is_selected = True
                    break

        # ul
        elif elements[self.index].tag_name == "ul":
            lis = elements[self.index].find_elements_by_tag_name('li')

            for li in lis:
                logger.step_normal("Element [%s]: li [%s]" % (self.__name__, li.text))

                if li.text == value:
                    li.click()
                    is_selected = True
                    break

        # not support
        else:
            logger.step_fail("Element [%s]: Tag [%s] NOT support [Select] method" % (self.__name__, elements[self.index].tag_name))

        if is_selected is False:
            logger.step_fail("No item selected!")

    @CompatibleMethod
    def SelectByOrder(self, order):
        logger.step_normal("Element [%s]: Select by Order [%s]" % (self.__name__, order))

        order = int(order)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        # ul
        if elements[self.index].tag_name == "ul":
            lis = elements[self.index].find_elements_by_tag_name('li')

            if order > 0:

                # wait and try more times if NO item found.
                t = 0
                while len(lis) == 0:
                    lis = elements[self.index].find_elements_by_tag_name('li')
                    time.sleep(3)
                    t = t + 1
                    logger.step_normal("Element [%s]: Wait 3 Seconds for [li]" % self.__name__)

                    if t == 20 and len(lis) == 0:
                        logger.step_fail("Element [%s]: List Count = [%s]." % (self.__name__, len(lis)))
                        return

                logger.step_normal("Element [%s]: List Count = [%s]." % (self.__name__, len(lis)))

                if order > len(lis):
                    logger.step_fail("Element [%s]: Not so many lists. [%s]" % (self.__name__, len(lis)))
                else:
                    action = webdriver.ActionChains(Browser.RunningBrowser)

                    # Added to avoid error: "Element is no longer attached to the DOM"
                    elements = Browser.RunningBrowser.find_elements(self.by, self.value)

                    logger.step_normal("Element [%s]: Do Click [%s]" % (self.__name__, lis[order-1].text))
                    lis[order-1].click()

                    """
                    lis = elements[self.index].find_elements_by_tag_name('li')
                    action.click(lis[order-1])
                    action.perform()
                    """

            else:
                logger.step_fail("Order = [%s], Value Error." % order)


        # select
        # if elements[self.index].tag_name == "select":
        else:
            options = elements[self.index].find_elements_by_tag_name('option')

            if order > 0:

                # wait and try more times if NO item found.
                t = 0
                while len(options) == 0:
                    options = elements[self.index].find_elements_by_tag_name('option')
                    time.sleep(3)
                    t = t + 1
                    logger.step_normal("Element [%s]: Wait 3 Seconds for [option]" % self.__name__)

                    if t == 20 and len(options) == 0:
                        logger.step_fail("Element [%s]: options Count = [%s]." % (self.__name__, len(options)))
                        return

                logger.step_normal("Element [%s]: options Count = [%s]." % (self.__name__, len(options)))

                if order > len(options):
                    logger.step_fail("Element [%s]: Not so many options. [%s]" % (self.__name__, len(options)))
                else:
                    logger.step_normal("Element [%s]: Do Click [%s]" % (self.__name__, options[order-1].text))
                    options[order-1].click()

                    """
                    action = webdriver.ActionChains(Browser.RunningBrowser)
                    action.click()
                    action.perform()
                    """

            else:
                logger.step_fail("Order = [%s], Value Error." % order)

    @CompatibleMethod
    def SelectByPartText(self, value):
        logger.step_normal("Element [%s]: Select [%s]." % (self.__name__, value))

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        is_selected = False

        # select
        if elements[self.index].tag_name == "select":
            options = elements[self.index].find_elements_by_tag_name('option')

            for option in options:
                if value in option.text:
                    logger.step_normal("Element [%s]: Select [%s]." % (self.__name__, option.text))
                    option.click()
                    is_selected = True
                    break

        # ul
        elif elements[self.index].tag_name == "ul":
            lis = elements[self.index].find_elements_by_tag_name('li')

            for li in lis:
                if value in li.text:
                    logger.step_normal("Element [%s]: Select [%s]." % (self.__name__, li.text))
                    li.click()
                    is_selected = True
                    break

        # not support
        else:
            logger.step_fail("Element [%s]: Tag [%s] NOT support [Select] method" % (self.__name__, elements[self.index].tag_name))

        if is_selected is False:
            logger.step_fail("No item selected!")

    @CompatibleMethod
    def SendEnter(self):
        logger.step_normal("Element [%s]: SendEnter()" % self.__name__, )

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        action = webdriver.ActionChains(Browser.RunningBrowser)
        action.send_keys_to_element(elements[self.index], Keys.ENTER)
        action.perform()

    @CompatibleMethod
    def Set(self, value):
        logger.step_normal("Element [%s]: Set [%s]." % (self.__name__, value))

        value = str(value)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        if elements[self.index].tag_name == "select" or elements[self.index].tag_name == "ul":
            self.Select(value)

        else:
            elements[self.index].clear()
            elements[self.index].send_keys(value)

            """
            elements[self.index].clear()
            action = webdriver.ActionChains(Browser.RunningBrowser)
            action.send_keys_to_element(elements[self.index], value)
            action.perform()
            """

    @CompatibleMethod
    def TypeInWithoutClear(self, value):
        """Input value without clear existing values"""

        logger.step_normal("Element [%s]: TypeInWithoutClear [%s]." % self.__name__, value)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        elements[self.index].send_keys(value)

    @CompatibleMethod
    def VerifyAttribute(self, attribute, value, assertion='equal'):
        """
        Example:
            NewClaim.Dates.ReminderDate.VerifyAttribute('ng-model', 'hello', assertion='equal')
            NewClaim.Dates.ReminderDate.VerifyAttribute('ng-model', 'hello', assertion='contain')
            NewClaim.Dates.ReminderDate.VerifyAttribute('ng-model', 'hello', assertion='in')
            NewClaim.Dates.ReminderDate.VerifyAttribute('ng-model', 'hello', assertion='not equal')

        :param assertion:
            in          => Real value [in] 'hello'. For example: real_value is 'he'
            equal       => Real value [equal] 'hello'. For example: real_value is 'hello'
            contain     => Real value [contain] 'hello'. For example: real_value is 'hello world'.
            not equal   => Real value [not equal] 'hello'. For example: real_value is 'hallow'
            not contain => Real value [not contain] 'hello'. For example: real_value is 'hi world'.
        """
        logger.step_normal("Element [%s]: VerifyAttribute [%s] <%s> [%s]." % (self.__name__, attribute, assertion, value))

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)
        real_value = elements[self.index].get_attribute(attribute)

        if assertion.lower() == 'equal':
            if value == real_value:
                logger.step_pass("real value=[%s]" % real_value)
            else:
                logger.step_fail("real value=[%s]" % real_value)

        elif assertion.lower() == 'not equal':
            if not value == real_value:
                logger.step_pass("real value=[%s]" % real_value)
            else:
                logger.step_fail("real value=[%s]" % real_value)

        elif assertion.lower() == 'contain':
            if value in real_value:
                logger.step_pass("real value=[%s]" % real_value)
            else:
                logger.step_fail("real value=[%s]" % real_value)

        elif assertion.lower() == 'not contain':
            if not value in real_value:
                logger.step_pass("real value=[%s]" % real_value)
            else:
                logger.step_fail("real value=[%s]" % real_value)

        elif assertion.lower() == 'in':
            if real_value in value:
                logger.step_pass("real value=[%s]" % real_value)
            else:
                logger.step_fail("real value=[%s]" % real_value)

        else:
            logger.step_fail("code error.")

    @CompatibleMethod
    def VerifyEnabled(self, trueOrfalse):
        logger.step_normal("Element [%s]: Verify Enabled = [%s]" % (self.__name__, trueOrfalse))

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        is_disabled = elements[self.index].get_attribute("disabled")
        logger.step_normal("Element [%s]: attribute 'is_disabled' = [%s]" % (self.__name__, is_disabled))

        if is_disabled == "true":
            if trueOrfalse is False:
                logger.step_pass("Pass...")
            else:
                logger.step_fail("Fail...")

        elif elements[self.index].is_enabled():
            if trueOrfalse is True:
                logger.step_pass("Pass")
            else:
                logger.step_fail("Fail")

        else:
            logger.step_fail("Not verified.")

    @CompatibleMethod
    def VerifyExistence(self, trueORfalse):
        """
        EXAMPLE
            Page.Element.VerifyExistence(True)

        :param trueORfalse:  True or False
        :return:
        """
        logger.step_normal("Element [%s]: Verify Existence = [%s]." % (self.__name__, trueORfalse))

        if trueORfalse is True:
            self.__wait_for_appearing()
        else:
            self.__wait_for_disappearing()

        elements = Browser.RunningBrowser.find_elements(self.by, self.value)
        logger.step_normal("Element [%s]: Count = [%s]" % (self.__name__, len(elements)))



        if len(elements) > 0:
            if trueORfalse is True:
                logger.step_pass("Exist!")
            else:
                logger.step_fail("Exist!")
        else:
            if trueORfalse is False:
                logger.step_pass("Not Exist!")
            else:
                logger.step_fail("Not Exist!")

    @CompatibleMethod
    def VerifyInnerHTMLContains(self, content):
        self.VerifyAttribute("innerHTML", content, assertion="contain")

    @CompatibleMethod
    def VerifyVisible(self, trueORfalse):
        logger.step_normal("Element [%s]: Verify Visible = [%s]." % (self.__name__, trueORfalse))

        self.__wait()

        elements = Browser.RunningBrowser.find_elements(self.by, self.value)
        logger.step_normal("Element [%s]: Count = [%s]" % (self.__name__, len(elements)))

        if elements[self.index].is_displayed():
            if trueORfalse is True:
                logger.step_pass("Visible!")
            else:
                logger.step_fail("Visible!")
        else:
            if trueORfalse is False:
                logger.step_pass("Not Visible!")
            else:
                logger.step_fail("Not Visible!")

    @CompatibleMethod
    def WaitForAppearing(self):
        logger.step_normal("Element [%s]: WaitForAppearing..." % self.__name__)

        self.__wait_for_appearing()

    @CompatibleMethod
    def WaitForAttribute(self, attribute, value, assertion="equal"):
        """
        Example:
            NewClaim.Dates.ReminderDate.WaitForAttribute('ng-model', 'hello', assertion='equal')
            NewClaim.Dates.ReminderDate.WaitForAttribute('ng-model', 'hello', assertion='contain')
            NewClaim.Dates.ReminderDate.WaitForAttribute('ng-model', 'hello', assertion='not contain')
            NewClaim.Dates.ReminderDate.WaitForAttribute('ng-model', 'hello', assertion='in')
            NewClaim.Dates.ReminderDate.WaitForAttribute('ng-model', 'hello', assertion='not equal')

        :param assertion:
            in          => Real value [in] 'hello'. For example: real_value is 'he'
            equal       => Real value [equal] 'hello'. For example: real_value is 'hello'
            contain     => Real value [contain] 'hello'. For example: real_value is 'hello world'.
            not equal   => Real value [not equal] 'hello'. For example: real_value is 'hallow'
            not contain => Real value [not contain] 'hello'. For example: real_value is 'hi world'.
        """
        logger.step_normal("Element [%s]: WaitForAttribute [%s] <%s> [%s]." % (self.__name__, attribute, assertion, value))

        i = 0
        while True:
            self.__wait()
            elements = Browser.RunningBrowser.find_elements(self.by, self.value)

            real_value = elements[self.index].get_attribute(attribute)

            if assertion.lower() == 'equal':
                if value.lower() == real_value.lower():
                    logger.step_normal("Yes! real value=[%s]" % real_value)
                    break
                else:
                    logger.step_normal("No! real value=[%s]" % real_value)

            elif assertion.lower() == 'contain':
                if value.lower() in real_value.lower():
                    logger.step_normal("Yes! real value=[%s]" % real_value[:150])
                    break
                else:
                    logger.step_normal("No! real value=[%s]" % real_value[:150])

            elif assertion.lower() == 'not contain':
                if value.lower() in real_value.lower():
                    logger.step_normal("Yes! real value=[%s]" % real_value[:150])
                else:
                    logger.step_normal("No! real value=[%s]" % real_value[:150])
                    break

            elif assertion.lower() == 'in':
                if real_value.lower() in value.lower():
                    logger.step_normal("Yes! real value=[%s]" % real_value[:150])
                    break
                else:
                    logger.step_normal("No! real value=[%s]" % real_value[:150])

            elif assertion.lower() == 'not equal':
                if value.lower() == real_value.lower():
                    logger.step_normal("No! real value=[%s]" % real_value)
                else:
                    logger.step_normal("Yes! real value=[%s]" % real_value)
                    break

            else:
                logger.step_fail("code error.")

            i = i + 1
            if i > 90:
                logger.step_fail("Not Found Expected Value! real value=[%s]" % real_value)
                break

            time.sleep(1)

    @CompatibleMethod
    def WaitForDisappearing(self):
        logger.step_normal("Element [%s]: WaitForDisappearing..." % self.__name__)

        self.__wait_for_disappearing()

    @CompatibleMethod
    def WaitForEnabled(self):
        logger.step_normal("Element [%s]: WaitForEnabled..." % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        t = 0
        while t < 90:
            if elements[self.index].is_enabled():
                logger.step_normal("Element [%s]: is enabled now." % self.__name__)
                break
            else:
                logger.step_normal("Element [%s]: still NOT enabled, wait 1 second." % self.__name__)
                time.sleep(1)

            t = t + 1

    @CompatibleMethod
    def WaitForVisible(self):
        logger.step_normal("Element [%s]: WaitForVisible..." % self.__name__)

        self.__wait()
        elements = Browser.RunningBrowser.find_elements(self.by, self.value)

        t = 0
        while t < 90:
            if elements[self.index].is_displayed():
                logger.step_normal("Element [%s]: IS visible now." % self.__name__)
                break
            else:
                logger.step_normal("Element [%s]: Still NOT visible, wait 1 second." % self.__name__)
                time.sleep(1)










import os
from datetime import datetime

from knitter import library
from knitter.configure import General


def html_source_header(title="Web UI Automation Test Report"):
    return """
     <!DOCTYPE HTML>
        <html>

        <head>
            <title>%s</title>

            <style>
                a {
                    color: #034CAC;
                    overflow: hidden;
                    text-decoration: none;
                    display: inline; /* do not show the dot of list */
                }

                a:hover {
                    text-decoration: underline;
                    color: #034CAC;
                }
                a:visited {
                    color: #034CAC;
                }
                body {
                    width: 100%%;
                    color: #5a5a5a;
                    font-size: 13px;
                    font-family: Verdana, Courier New;
                    text-align: center;
                }
                table, td, th {
                    border: 1px solid rgb(119, 119, 119);
                    border-collapse: collapse;
                    width: 1400px;
                    text-align: center;
                }
                .tcaption {
                    background-color: rgb(116, 162, 179);
                }
                .theader {
                    background-color: rgba(186, 216, 221, 0.61);
                }
                .tfail{
                    background-color: #FF6F6F;
                }
                h1{
                    text-align: center;
                }
                table {
                    box-shadow: 3px 3px 2px #d3d3d3;
                }
                
                .report-link{
                    color: #5a5a5a !important;
                    font-family: Courier New, Courier, monospace;
                    background-color:#f7f7f7; 
                    border-radius: 2px; 
                    box-shadow: 2px 2px 2px #d3d3d3;
                    margin: 10px;
                    padding: 5px 20px 5px 20px;
                }
                a.report-link:hover{
                    font-family: Courier New, Courier, monospace;
                    color: #5a5a5a;
                    background-color:#f1f1f1; 
                    text-decoration: none;
                    box-shadow: 4px 4px 2px #d3d3d3;
                    border-radius: 2px; 
                }
                hr{
                    margin: 20px;
                }
            </style>
        </head>    
    """ % title


def html_source_body(title="Web UI Automation Test Report", countdown=True):
    if countdown is False:
        return """
        <body> 
        <h1>%s</h1> 
        <p style="text-align: center">
            <a href="..\index.html" class="report-link">Latest Test Report</a>
            &nbsp;&nbsp;&nbsp;
            <a href="..\history.html" class="report-link">History Test Reports</a>
        </p>
        <br />
        """ % title
    
    return """
    <body>
    <script>
        function countdown(){
            var timer = 9;
            setInterval(function() {
                timer--;
                document.getElementById('timer').innerHTML = timer;
        
            }, 1000);
        }
        function autorefresh(){
            window.location.reload(); 
        } 
        
        countdown();
        window.setTimeout('autorefresh()', 9000);
    </script>
    
    <h1>%s</h1>
    <p style="text-align: center">
        <a href="index.html" class="report-link">Latest Test Report</a>
        &nbsp;&nbsp;&nbsp;
        <a href="history.html" class="report-link">History Test Reports</a>
    </p>
    <br />
    
""" % title


def html_source_table1(args):
    return """
    <table style="margin: auto">
        <tr class="tcaption"><th colspan="5">Summary</th></tr>
        
        <tr class="theader">
            <th>Start Time/Stop Time</th>
            <th>Duration</th>
            <th>Total Cases</th>
            <th>Passed Cases</th>
            <th>Failed Cases</th>
        </tr>
        <tr>
            <td style='width: 27%%'>%s =&gt; %s</td>
            <td>%s</td>
            <td>%s</td>
            <td>%s</td>
            <td>%s</td>
        </tr>
    </table>
""" % tuple(args)


def html_source_table_history_header():
    return """
    <table style="margin: auto">
        <tr class="tcaption"><th colspan="5">History Reports</th></tr>
        
        <tr class="theader">
            <th>Folder</th>
            <th>Duration</th>
            <th>Total Cases</th>
            <th>Passed Cases</th>
            <th>Failed Cases</th>
        </tr>
""" 


def html_source_table_history(*args):
    return """
        <tr>
            <td style='width: 50%%; text-align: left; padding-left: 20px;'><a href="%s/index.html">%s</a></td>
            <td>%s</td>
            <td>%s</td>
            <td>%s</td>
            <td>%s</td>
        </tr>
""" % args


def html_source_table2():
    return """
    <br />
    <br />
    <table style="margin: auto">
        <tr class="tcaption"><th colspan="5">Details</th></tr>
        <tr class="theader">
            <th>Start Time/Stop Time</th>
            <th>Test Case</th>
            <th>Duration</th>
            <th>Browser</th>
            <th>Result</th>
        </tr>

"""


def html_source_test_cases(test_cases):
    return_src_code = ""
    
    for test_case in test_cases:
        return_src_code += """
                <tr>
                    <td style="width: 20%%">%s</td>
                    <td style="width: 52%%; text-align: left;">&nbsp;%s</td>
                    <td>%s</td>
                    <td>%s</td>
                    %s
                </tr>
        """ % tuple(test_case)
    
    return return_src_code


def html_source_end_table():
    return """
    </table> 
<br />
<br />
<br />
"""


def html_source_version_info():
    return u"""
<p style="text-align: center">%s</p>
""" % library.version_info()


def html_source_foot():
    return u"""
<hr />
<p style="text-align: center">@2014-2018 Powered by <a href="https://pypi.org/project/knitter">Knitter</a></p>
<br />
</body>
</html>
"""


def generate_html_report(test_status, test_cases=[], countdown=True):
    if General.QuickTest is True:
        return
    
    library.create_folder(General.Path.Result)
    
    with open(os.path.join(General.Path.Result, "index.html"), "w") as f:
        f.write(html_source_header())
        f.write(html_source_body(countdown=countdown))
        
        f.write(html_source_table1(test_status))
        f.write(html_source_table2())
         
        f.write(html_source_test_cases(test_cases))
        
        f.write(html_source_end_table())
        f.write(html_source_version_info())
        f.write(html_source_foot())


def save_current_report_to_repository():
    if General.QuickTest is True:
        return

    report_dir = os.path.join(General.Path.Result,
                              "%s__%s" % (datetime.strptime(General.Total.StartTime, "%Y-%m-%d %H:%M:%S").strftime("%Y-%m-%d_%H%M%S"),
                                         datetime.strptime(General.Total.EndTime, "%Y-%m-%d %H:%M:%S").strftime("%Y-%m-%d_%H%M%S")))
    library.delete_folder(report_dir)
    library.create_folder(report_dir)

    if os.path.exists(os.path.join(General.Path.Result, "testcase")):
        library.copy(os.path.join(General.Path.Result, "testcase"), os.path.join(report_dir, "testcase"))

    if os.path.exists(os.path.join(General.Path.Result, "screenshots")):
        library.copy(os.path.join(General.Path.Result, "screenshots"), os.path.join(report_dir, "screenshots"))

    if os.path.exists(os.path.join(General.Path.Result, "index.html")):
        library.copy(os.path.join(General.Path.Result, "index.html"), report_dir)
    
    with open(os.path.join(report_dir, "status.ini"), "w") as f:
        f.write("Duration=%s\n" % str(datetime.strptime(General.Total.EndTime, "%Y-%m-%d %H:%M:%S") -
                                      datetime.strptime(General.Total.StartTime, "%Y-%m-%d %H:%M:%S")))
        f.write("TotalCases=%s\n" % str(General.Total.NumberOfTestCasePass + General.Total.NumberOfTestCaseFail))
        f.write("PassedCases=%s\n" % str(General.Total.NumberOfTestCasePass))
        f.write("FailedCases=%s\n" % str(General.Total.NumberOfTestCaseFail))


def generate_report_history():
    if General.QuickTest is True:
        return

    folders = library.get_sub_folder_names(General.Path.Result)
    
    reports = []
    for folder in folders:
        if folder.startswith("testcase") or folder.startswith("screenshot"):
            continue
        reports.append(folder)

    with open(os.path.join(General.Path.Result, "history.html"), "w") as f:
        f.write(html_source_header(title="Web UI Automation Test History Reports"))
        f.write(html_source_body(title="Web UI Automation Test History Reports"))
        f.write(html_source_table_history_header())
        
        i = 0
        for report in sorted(reports, reverse=True):
            
            Duration = library.get_value_from_conf(os.path.join(General.Path.Result, report, "status.ini"), "Duration")
            TotalCases = library.get_value_from_conf(os.path.join(General.Path.Result, report, "status.ini"), "TotalCases")
            PassedCases = library.get_value_from_conf(os.path.join(General.Path.Result, report, "status.ini"), "PassedCases")
            FailedCases = library.get_value_from_conf(os.path.join(General.Path.Result, report, "status.ini"), "FailedCases")
            
            f.write(html_source_table_history(report, report, Duration, TotalCases, PassedCases, FailedCases))
            
            i = i + 1

        f.write(html_source_end_table())
        f.write(html_source_foot())


if __name__ == "__main__":
    General.Path.Result = "D:/"
    generate_report_history()







